@@ -15,8 +15,11 @@ class TestMemory(unittest.TestCase):
         memory = MPI.memory
         mem = memory()
         self.assertEqual(mem.address, 0)
+        self.assertEqual(mem.obj, None)
         self.assertEqual(mem.nbytes, 0)
-        self.assertFalse(mem.readonly)
+        self.assertEqual(mem.readonly, False)
+        self.assertEqual(mem.format, 'B')
+        self.assertEqual(mem.itemsize, 1)
         self.assertEqual(len(mem), 0)
         mem[:] = 0
         mem[:] = memory()
@@ -37,14 +40,69 @@ class TestMemory(unittest.TestCase):
         mem.release()
         self.assertEqual(mem.address, 0)
         self.assertEqual(mem.nbytes, 0)
-        self.assertFalse(mem.readonly)
+        self.assertEqual(mem.readonly, False)
+
+    def testNewBad(self):
+        memory = MPI.memory
+        for obj in (None, 0, 0.0, [], (), []):
+            self.assertRaises(TypeError,  memory, obj)
+
+    def testNewBytes(self):
+        memory = MPI.memory
+        obj = b"abc"
+        mem = memory(obj)
+        self.assertEqual(mem.obj, obj)
+        self.assertEqual(mem.nbytes, len(obj))
+        self.assertEqual(mem.readonly, True)
+
+    def testNewBytearray(self):
+        memory = MPI.memory
+        obj = bytearray([1,2,3])
+        mem = memory(obj)
+        self.assertEqual(mem.obj, obj)
+        self.assertEqual(mem.nbytes, len(obj))
+        self.assertEqual(mem.readonly, False)
+
+    @unittest.skipIf(array is None, 'array')
+    def testNewArray(self):
+        memory = MPI.memory
+        obj = array('i', [1,2,3])
+        mem = memory(obj)
+        self.assertEqual(mem.obj, obj)
+        self.assertEqual(mem.nbytes, len(obj)*obj.itemsize)
+        self.assertEqual(mem.readonly, False)
+
+    def testAllocate(self):
+        memory = MPI.memory
+        for size in (0, 1, 2):
+            mem = memory.allocate(size)
+            self.assertEqual(mem.nbytes, size)
+            self.assertNotEqual(mem.address, 0)
+        for clear in (False, True):
+            mem = memory.allocate(1024, clear)
+            self.assertEqual(mem.nbytes, 1024)
+            self.assertNotEqual(mem.address, 0)
+            if clear:
+                self.assertEqual(mem[0], 0)
+                self.assertEqual(mem[-1], 0)
+        self.assertRaises(TypeError,  memory.allocate, None)
+        self.assertRaises(ValueError, memory.allocate, -1)
+
+    def testFromBufferBad(self):
+        memory = MPI.memory
+        for obj in (None, 0, 0.0, [], (), []):
+            self.assertRaises(TypeError,  memory.frombuffer, obj)
 
     def testFromBufferBytes(self):
         memory = MPI.memory
         mem = memory.frombuffer(b"abc", readonly=True)
         self.assertNotEqual(mem.address, 0)
+        self.assertEqual(type(mem.obj), bytes)
+        self.assertEqual(mem.obj, b"abc")
         self.assertEqual(mem.nbytes, 3)
-        self.assertTrue(mem.readonly)
+        self.assertEqual(mem.readonly, True)
+        self.assertEqual(mem.format, 'B')
+        self.assertEqual(mem.itemsize, 1)
         self.assertEqual(len(mem), 3)
         if sys.version_info < (3,0):
             b = buffer(mem)
@@ -63,7 +121,7 @@ class TestMemory(unittest.TestCase):
         mem.release()
         self.assertEqual(mem.address, 0)
         self.assertEqual(mem.nbytes, 0)
-        self.assertFalse(mem.readonly)
+        self.assertEqual(mem.readonly, False)
 
     @unittest.skipIf(array is None, 'array')
     def testFromBufferArrayRO(self):
@@ -71,8 +129,11 @@ class TestMemory(unittest.TestCase):
         obj = array('B', [1,2,3])
         mem = memory.frombuffer(obj, readonly=True)
         self.assertNotEqual(mem.address, 0)
+        self.assertEqual(type(mem.obj), array)
         self.assertEqual(mem.nbytes, 3)
-        self.assertTrue(mem.readonly)
+        self.assertEqual(mem.readonly, True)
+        self.assertEqual(mem.format, 'B')
+        self.assertEqual(mem.itemsize, 1)
         self.assertEqual(len(mem), 3)
         if sys.version_info < (3,0):
             b = buffer(mem)
@@ -91,7 +152,7 @@ class TestMemory(unittest.TestCase):
         mem.release()
         self.assertEqual(mem.address, 0)
         self.assertEqual(mem.nbytes, 0)
-        self.assertFalse(mem.readonly)
+        self.assertEqual(mem.readonly, False)
 
     @unittest.skipIf(array is None, 'array')
     def testFromBufferArrayRW(self):
@@ -100,7 +161,7 @@ class TestMemory(unittest.TestCase):
         mem = memory.frombuffer(obj, readonly=False)
         self.assertNotEqual(mem.address, 0)
         self.assertEqual(mem.nbytes, 3)
-        self.assertFalse(mem.readonly)
+        self.assertEqual(mem.readonly, False)
         self.assertEqual(len(mem), 3)
         if sys.version_info < (3,0):
             b = buffer(mem)
@@ -125,7 +186,7 @@ class TestMemory(unittest.TestCase):
         mem.release()
         self.assertEqual(mem.address, 0)
         self.assertEqual(mem.nbytes, 0)
-        self.assertFalse(mem.readonly)
+        self.assertEqual(mem.readonly, False)
 
     @unittest.skipIf(array is None, 'array')
     def testFromAddress(self):
@@ -136,7 +197,7 @@ class TestMemory(unittest.TestCase):
         mem = memory.fromaddress(addr, nbytes, readonly=False)
         self.assertNotEqual(mem.address, 0)
         self.assertEqual(mem.nbytes, 3)
-        self.assertFalse(mem.readonly)
+        self.assertEqual(mem.readonly, False)
         self.assertEqual(len(mem), 3)
         if sys.version_info < (3,0):
             b = buffer(mem)
@@ -161,7 +222,19 @@ class TestMemory(unittest.TestCase):
         mem.release()
         self.assertEqual(mem.address, 0)
         self.assertEqual(mem.nbytes, 0)
-        self.assertFalse(mem.readonly)
+        self.assertEqual(mem.readonly, False)
+
+    def testToReadonly(self):
+        memory = MPI.memory
+        obj = bytearray(b"abc")
+        mem1 = memory.frombuffer(obj)
+        mem2 = mem1.toreadonly()
+        self.assertEqual(mem1.readonly, False)
+        self.assertEqual(mem2.readonly, True)
+        self.assertEqual(mem1.address, mem2.address)
+        self.assertEqual(mem1.obj, mem2.obj)
+        self.assertEqual(type(mem1.obj), type(mem2.obj))
+        self.assertEqual(mem1.nbytes, mem2.nbytes)
 
     def testSequence(self):
         n = 16
@@ -173,7 +246,7 @@ class TestMemory(unittest.TestCase):
             self.assertTrue(type(mem) is MPI.memory)
             self.assertTrue(mem.address != 0)
             self.assertEqual(mem.nbytes, n)
-            self.assertFalse(mem.readonly)
+            self.assertEqual(mem.readonly, False)
             self.assertEqual(len(mem), n)
             def delitem():  del mem[n]
             def getitem1(): return mem[n]
@@ -194,6 +267,11 @@ class TestMemory(unittest.TestCase):
             for i in range(n):
                 self.assertEqual(mem[i], i)
             mem[:] = 0
+            for i in range(-n, 0):
+                mem[i] = abs(i)
+            for i in range(-n, 0):
+                self.assertEqual(mem[i], abs(i))
+            mem[:] = 0
             for i in range(n):
                 self.assertEqual(mem[i], 0)
             mem[:] = 255
@@ -223,7 +301,7 @@ class TestMemory(unittest.TestCase):
             MPI.Free_mem(mem)
             self.assertEqual(mem.address, 0)
             self.assertEqual(mem.nbytes, 0)
-            self.assertFalse(mem.readonly)
+            self.assertEqual(mem.readonly, False)
 
 try:
     MPI.memory
